@@ -426,7 +426,7 @@ GdipGetPathData (GDIPCONST GpPath *path, GpPathData *pathData)
 
         pathData->Types = g_byte_array_to_array (path->types);
 	if (!pathData->Types) {
-		g_array_free (pathData->Points, TRUE);
+		GdipFree (pathData->Points);
 		pathData->Points = NULL;
 		return OutOfMemory;
 	}
@@ -569,7 +569,6 @@ GpStatus
 GdipReversePath (GpPath *path)
 {
 	int length, i;
-	GByteArray *types;
 	GArray *points;
 
 	if (!path)
@@ -580,18 +579,19 @@ GdipReversePath (GpPath *path)
 	if (length <= 1)
 		return Ok;
 
-	types = g_byte_array_sized_new (length);
-	points = g_array_sized_new (FALSE, TRUE, sizeof (GpPointF), length);
+	/* NOTE: PathTypes are NOT reversed */
 
-        for (i = length; i > 0; i--) {
-                byte t = g_array_index (path->types, byte, i);
+	/* FIXME: this could be done without allocations */
+	points = g_array_sized_new (FALSE, TRUE, sizeof (GpPointF), length);
+	if (!points)
+		return OutOfMemory;
+
+        for (i = length - 1; i >= 0; i--) {
                 GpPointF pt = g_array_index (path->points, GpPointF, i);
-                
-                g_byte_array_append (types, &t, 1);
                 g_array_append_val (points, pt);
         }
+	g_array_free (path->points, TRUE);
         path->points = points;
-        path->types = types;
         
         return Ok;
 }
@@ -691,8 +691,7 @@ append_arcs (GpPath *path, float x, float y, float width, float height, float st
 	int i;
 	float drawn = 0;
 	float endAngle = startAngle + sweepAngle;
-	int sign = (endAngle > 0) ? 1 : -1;
-	int increment = sign * 90;
+	int increment = (endAngle > 0) ? 90 : -90;
 	bool enough = FALSE;
 
 	if (fabs (sweepAngle) >= 360) {
@@ -716,6 +715,10 @@ append_arcs (GpPath *path, float x, float y, float width, float height, float st
 			enough = TRUE;
 		}
 
+		/* a near zero value will introduce bad artefact in the drawing (#78999) */
+		if (gdip_near_zero (additional))
+			return;
+
 		append_arc (path,
 			    (i == 0) ? TRUE : FALSE,  /* only move to the starting pt in the 1st iteration */
 			    x, y, width, height,      /* bounding rectangle */
