@@ -436,16 +436,43 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 			for (i = 0; i < nlines; i++) {
 				int j;
 				guchar *inptr, *outptr;
+				int width = result->active_bitmap->width;
 
-				inptr = lines[i] + (result->active_bitmap->width) * 3;
+				inptr = lines[i] + (width * 3);
 				outptr = lines[i] + stride;
-				for (j = 0; j < result->active_bitmap->width; j++) {
+				/* this loop can't work for the last two lines as it will overwrite values it requires */
+				for (j = 0; j < width - 2; j++) {
 					/* Note the swapping of R and B, to get ARGB from what
 					 * looks like BGR data. */
 					inptr -= 3;
 					outptr -= 4;
 					set_pixel_bgra(outptr, 0, inptr[2], inptr[1], inptr[0], 0xff);
 				}
+				/* keep last 2 lines in temporary variables */
+				if (width > 1) {
+					byte b2, g2, r2, b1, g1, r1;
+					inptr -= 3;
+					b2 = inptr[2];
+					g2 = inptr[1];
+					r2 = inptr[0];
+					inptr -= 3;
+					b1 = inptr[2];
+					g1 = inptr[1];
+					r1 = inptr[0];
+					outptr -= 4;
+					set_pixel_bgra(outptr, 0, b2, g2, r2, 0xff);
+					outptr -= 4;
+					set_pixel_bgra(outptr, 0, b1, g1, r1, 0xff);
+				} else 	if (width > 0) {
+					/* in case the jpeg has a single line */
+					byte b, g, r;
+					inptr -= 3;
+					b = inptr[2];
+					g = inptr[1];
+					r = inptr[0];
+					outptr -= 4;
+					set_pixel_bgra(outptr, 0, b, g, r, 0xff);
+				}
 			}
 		}
 	}
@@ -820,9 +847,7 @@ gdip_save_jpeg_image_to_file (FILE *fp, GpImage *image, GDIPCONST EncoderParamet
 }
 
 GpStatus
-gdip_load_jpeg_image_from_stream_delegate (GetBytesDelegate getBytesFunc,
-                                           SeekDelegate seeknFunc,
-                                           GpImage **image)
+gdip_load_jpeg_image_from_stream_delegate (dstream_t *loader, GpImage **image)
 {
     *image = NULL;
     return UnknownImageFormat;
