@@ -26,6 +26,7 @@
 #include "gdip.h"
 #include "gdipImage.h"
 #include "texturebrush.h"
+#include "matrix.h"
 
 static GpStatus gdip_texture_setup (GpGraphics *graphics, GpBrush *brush);
 static GpStatus gdip_texture_clone (GpBrush *brush, GpBrush **clonedBrush);
@@ -429,7 +430,6 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 	unsigned int	height;
 	GpStatus	status = Ok;
 	BOOL		dispose_bitmap;
-	GpMatrix	*product;
 
 	if ((graphics == NULL) || (brush == NULL) || (graphics->ct == NULL)) {
 		return InvalidParameter;
@@ -504,53 +504,20 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 		return GenericError;
 	}
 
-	GdipCreateMatrix (&product);
-
-	/* FIXME Cairo Bug: REPEAT and transformation do not go together.  */
-	/* To work around this we can create an intermediate surface first */
-	/* with REPEAT and use that with transformation as a pattern. But, */
-	/* that gets another problem of handling rotation. Since, we need  */
-	/* absolute rotation instead of rotation around 0,0.               */
-	/* There are two ways to get around this problem:                  */
-	/* 1. Uncommenting the following commented code once we know a     */
-	/* better way to handle the rotation axis problem.      OR         */
-	/* 2. Cairo bug gets fixed. Following commented code won't be      */
-	/* needed in that case.                                            */
-	/*
-		cairo_save (ct);
-		{
-			temp = cairo_surface_create_similar (cairo_get_target (ct),
-								format, width, height);
-			
-			if (temp == NULL) {
-				cairo_matrix_destroy (product);
-				return OutOfMemory;
-			}
-			
-			cairo_set_target_surface (ct, temp);
-			cairo_set_source (ct, texture->pattern);
-			cairo_rectangle (ct, 0, 0, width, height);
-			cairo_fill (ct);
-		}
-		cairo_restore (ct);
-
-		pattern = cairo_pattern_create_for_surface (temp);
-	*/
 	pattern = texture->pattern;
-
 	if (pattern != NULL) {
-		/* Use both the matrices */
-		cairo_matrix_multiply (product, &texture->matrix, graphics->copy_of_ctm);
-		cairo_matrix_invert (product);
-		cairo_pattern_set_matrix (pattern, product);
-		cairo_set_source (ct, pattern);
-		/*	cairo_pattern_destroy (pattern); */
-	}
-	status = gdip_get_status (cairo_status (ct));
-	/*	cairo_surface_destroy (temp); */
-	GdipDeleteMatrix (product);
+		/* got something to apply ? */
+		if (!gdip_is_matrix_empty (&texture->matrix)) {
+			cairo_matrix_t product;
 
-	return status;
+		        gdip_cairo_matrix_copy (&product, &texture->matrix);
+			cairo_matrix_invert (&product);
+			cairo_pattern_set_matrix (pattern, &product);
+		}
+		cairo_set_source (ct, pattern);
+	}
+
+	return gdip_get_status (cairo_status (ct));
 }
 
 GpStatus
@@ -587,13 +554,15 @@ gdip_texture_clone (GpBrush *brush, GpBrush **clonedBrush)
 	if (status != Ok) {
 		if (result->image)
 			GdipDisposeImage (result->image);
-		return status;
+		GdipFree (result);
+		result = NULL;
+	} else {
+		cairo_surface_reference (result->image->surface);
 	}
-	cairo_surface_reference (result->image->surface);
 
 	*clonedBrush = (GpBrush *) result;
 
-	return Ok;
+	return status;
 }
 
 GpStatus
@@ -659,6 +628,8 @@ GdipCreateTexture (GpImage *image, GpWrapMode wrapMode, GpTexture **texture)
 		if (result->image)
 			GdipDisposeImage (result->image);
 		cairo_surface_destroy (imageSurface);
+		GdipFree (result);
+		*texture = NULL;
 		return status;
 	}
 
@@ -739,11 +710,36 @@ GdipCreateTexture2I (GpImage *image, GpWrapMode wrapMode, int x, int y, int widt
 	}
 
 	result->image = NULL;
-	status = GdipCloneImage (image, &result->image);
+	if ((x == 0) && (y == 0)) {
+		/* the texture share the same dimension as the image, so we can clone it*/
+		status = GdipCloneImage (image, &result->image);
+	} else {
+		/* otherwise we need to make a copy of the requested (rectangle) region */
+		int new_stride = (int) (image->active_bitmap->stride * (image->active_bitmap->width - x) / image->active_bitmap->width);
+		int format = -1;
+		switch (image->cairo_format) {
+		case CAIRO_FORMAT_RGB24:
+			format = Format24bppRgb;
+			break;
+		case CAIRO_FORMAT_ARGB32:
+			format = Format32bppArgb;
+			break;
+		case CAIRO_FORMAT_A8:
+			format = Format8bppIndexed;
+			break;
+		case CAIRO_FORMAT_A1:
+			format = Format1bppIndexed;
+			break;
+		}
+		status = GdipCreateBitmapFromScan0 (width, height, new_stride, format, 
+			(unsigned char *)image->active_bitmap->scan0, &result->image);
+	}
 	if (status != Ok) {
 		if (result->image)
 			GdipDisposeImage (result->image);
 		cairo_surface_destroy (new);
+		GdipFree (result);
+		*texture = NULL;
 		return status;
 	}
 
@@ -877,28 +873,14 @@ GpStatus
 GdipRotateTextureTransform (GpTexture *texture, float angle, GpMatrixOrder order)
 {
 	GpStatus status;
-	GpPointF axis;
 
-	if (texture == NULL) {
+	if (texture == NULL)
 		return InvalidParameter;
-	}
 
-	/* Cairo uses origin (0,0) as the axis of rotation. However, we need 
-	 * to do absolute rotation. Following approach for shifting the axis
-	 * of rotation was suggested by Carl.
-	 */
+	status = GdipRotateMatrix (&texture->matrix, angle, order);
+	if (status == Ok)
+		texture->base.changed = TRUE;
 
-	/* Our pattern size is 2*(texture->rect) hence its centre is following */
-	axis.X = texture->rectangle.Width;
-	axis.Y = texture->rectangle.Height;
-
-	if ((status = GdipTranslateMatrix (&texture->matrix, -axis.X, -axis.Y, order)) == Ok) {
-		if ((status = GdipRotateMatrix (&texture->matrix, angle, order)) == Ok) {
-			if ((status = GdipTranslateMatrix (&texture->matrix, axis.X, axis.Y, order)) == Ok) {
-				texture->base.changed = TRUE;
-			}
-		}
-	}
 	return status;
 }
 
