@@ -24,7 +24,7 @@ gdip_load_tiff_image_from_stream_delegate (GetBytesDelegate getBytesFunc,
                                            GpImage **image);
 
 GpStatus 
-gdip_save_tiff_image_to_file (FILE *fp, GpImage *image,
+gdip_save_tiff_image_to_file (unsigned char *filename, GpImage *image,
 			      GDIPCONST EncoderParameters *params);
 
 GpStatus
